package psi.ip.gui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import javax.swing.UIManager;
import jpcap.*;
import jpcap.packet.*;

import psi.ip.net.Captor;
import psi.ip.net.FakeOptionsGenerator;
import psi.ip.net.IPv4Packet;
import psi.ip.net.Sender;

/**
 * Rozhrani mezi sitovou a prezentacni vrstvou.
 */
public class Controller implements Observer {

	public static final int MSG_CAPTURE_STOPPED = 0;
	private final List<IPv4Packet> packets;
	private PacketsTableModel model;
	private Captor captor;
	private Sender sender;
	private MainWindowFrame window;
	private NetworkInterface[] devices;
	private int currentDevice;
	private int maxPacketsCount = 0;
	private int tableSize = 0;
	private int packetsCounter = 0;
	private boolean fake;

	/**
	 * Konstruktor pro vytvoreni instance tridy.
	 */
	public Controller() {
		fake = false;
		devices = JpcapCaptor.getDeviceList();
		packets = Collections.synchronizedList(new ArrayList<IPv4Packet>()); //new ArrayList<IPv4Packet>();
		model = new PacketsTableModel(packets);
		captor = new Captor();
		sender = new Sender();

		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch (Exception e) {
			try {
				UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
			} catch (Exception f) {
				// nic se nedeje
			}
		}


		window = new MainWindowFrame(this);

		captor.addObserver(this);

		captor.setDevice(devices[0]);

		sender.setDevice(devices[0]);

		java.awt.EventQueue.invokeLater(new Runnable() {

			@Override
			public void run() {
				window.setVisible(true);
			}
		});

		window.setIsCapturing(false);
	}

	/**
	 * Spusti zachytavani paketu.
	 */
	public void startCapture() {
		if (maxPacketsCount == 0 || packetsCounter < maxPacketsCount) {
			window.setIsCapturing(true);
			captor.start();
		}
	}

	/**
	 * Zastavi zachytavani paketu.
	 */
	public void stopCapture() {
		captor.stop();
		window.setIsCapturing(false);
	}

	/**
	 * Vymaze tabulku zachycenych paketu.
	 */
	public void clear() {
		synchronized (packets) {
			packets.clear();
			packetsCounter = 0;
		}
		window.refreshTable();
	}

	/**
	 * Odesle paket. Paket je zapouzdren v Ethernet ramci,
	 * do ktereho se nastavi zadana zdrojova a cilova MAC adresa.
	 * @param packet Paket k odeslani.
	 * @param srcMac Zdrojova MAC adresa pro ulozeni do Ethernet ramce.
	 * @param destMac Cilova MAC adresa pro ulozeni do Ethernet ramce.
	 */
	public void sendPacket(Packet packet, byte[] srcMac, byte destMac[]) {
		sender.sendPacket(packet, srcMac, destMac);
	}

	/**
	 * Vrati seznam dostupnych sitovych rozhrani.
	 * @return Seznam dostupnych sitovych rozhrani.
	 */
	public String[] getDevices() {
		String[] deviceNames = new String[devices.length];
		for (int i = 0; i < devices.length; i++) {
			deviceNames[i] = devices[i].description;
		}
		return deviceNames;
	}

	/**
	 * Vrati tabulkovy model pro tabulku se zachycenymi pakety.
	 * @return Instance objektu <code>PacketsTableModel</code>.
	 */
	public PacketsTableModel getPacketsTableModel() {
		return model;
	}

	/**
	 * Nastavi zarizeni pouzite pro zachytavani a odesilani paketu.
	 * @param index Index zarizeni.
	 */
	public void setDevice(int index) {
		if (index >= 0 && index < devices.length) {
			currentDevice = index;
			captor.setDevice(devices[index]);
			sender.setDevice(devices[index]);
		} else {
			throw new IndexOutOfBoundsException("Invalid device index: " + index + ", expected between 0 and " + (devices.length - 1));
		}
	}

	/**
	 * Vrati MAC adresu aktualne nastaveneho sitoveho rozhrani.
	 * @return MAC adresa aktualne nastaveneho sitoveho rozhrani.
	 */
	public byte[] getCurrentDeviceMacAddress() {
		return devices[currentDevice].mac_address;
	}

	/**
	 * Nastavi limit poctu zachycenych paketu.
	 * @param count Pocet paketu, po jejichz zachyceni se zachytavani ukonci (0 pro neomezeny pocet).
	 */
	public void setPacketsCount(int count) {
		maxPacketsCount = count;
	}

	/**
	 * Nastavi pocet paketu, ktere se vejdou do tabulky.
	 * @param size Pocet paketu, ktere budou zobrazeny v tabulce.
	 */
	public void setTableSize(int size) {
		tableSize = size;
	}

	/**
	 * Vrati paket zadany indexem radku v tabulce.
	 * @param row Index paketu v tabulce.
	 * @return Paket z tabulky.
	 */
	public IPv4Packet getPacket(int row) {
		synchronized (packets) {
			return packets.get(row);
		}
	}

	/**
	 * Nastavi, zda se maji do zachycenych paketu vkladat falesne volitelne parametry IP.
	 * @param value <code>true</code>, pokud se maji vkladat falesne volitelne parametry.
	 */
	public void setFakeOptions(boolean value) {
		fake = value;
	}

	@Override
	public void update(Observable o, Object arg) {
		if (arg instanceof IPPacket) {

			IPPacket ipPacket = (IPPacket) arg;

			if (ipPacket.option == null) {
				byte[] opt = new byte[]{};
			}

			IPv4Packet ipv4Packet = new IPv4Packet((IPPacket) arg, packetsCounter);

			if (fake) {
				try {
					ipv4Packet.overrideOptions(FakeOptionsGenerator.getOptions());
				} catch (Exception ex) {
					System.out.println("Could not override packet options");
				}
			}

			synchronized (packets) {
				packets.add(ipv4Packet);

				packetsCounter++;

				// limit table size
				if (tableSize > 0) {
					while (packets.size() > tableSize) {
						packets.remove(0);
					}
				}
			}
			// limit captured packets count
			if (maxPacketsCount > 0 && packetsCounter >= maxPacketsCount) {
				stopCapture();
			}

		} else if (arg instanceof Integer) {
			int iarg = (Integer) arg;
			switch (iarg) {
				case MSG_CAPTURE_STOPPED:
					window.setIsCapturing(false);
					break;

				default:
			}
		} else {
			System.out.println("unknown object in Controller: " + arg.getClass().getCanonicalName());
		}

		synchronized (packets) {
			window.refreshTable();
		}
	}
}
