package psi.ip.net;

import java.net.InetAddress;
import jpcap.packet.IPPacket;

/**
 * Reprezentace IPv4 paketu.
 */
public class IPv4Packet {
	private IPPacket packet;

	private int headerLength;
	private byte typeOfService;
	private boolean reserved;
	private boolean dontFragment;
	private boolean moreFragments;
	private int checksum;
	private byte[] options;
	private int count;

	/**
	 * Vytvori instanci tridy.
	 * @param packet Instance tridy <code>IPPAcket</code>.
	 * @param count Poradi zachyceneho paketu.
	 */
	public IPv4Packet(IPPacket packet, int count) {
		this.packet = packet;
		this.count = count;
		
		if (packet.option == null) {
			options = new byte[0];
		} else {
			options = packet.option;
		}
		
		headerLength = (packet.header[14] & 0xf) * 4;
		typeOfService = packet.header[15];
		reserved = (packet.header[20] & (1 << 7)) != 0;
		dontFragment = (packet.header[20] & (1 << 6)) != 0;
		moreFragments = (packet.header[20] & (1 << 5)) != 0;
		checksum = (packet.header[24] & 0xff) << 8 | packet.header[25] & 0xff;
	}

	/**
	 * Vraci, v jakem poradi byl paket zachycen.
	 */
	public int getCount() {
		return count;
	}

	/**
	 * Vraci verzi IP protokolu.
	 */
	public int getVersion() {
		return packet.version;
	}

	/**
	 * Vraci delku hlavicky v bytech.
	 */
	public int getHeaderLength() {
		return headerLength;
	}

	/**
	 * Vraci oktet Type of Service.
	 */
	public byte getTypeOfService() {
		return typeOfService;
	}

	/**
	 * Vraci celkovou delku paketu.
	 */
	public int getTotalLength() {
		return packet.length + options.length;
	}

	/**
	 * Vraci hodnotu pole Identification.
	 */
	public int getIdentification() {
		return packet.ident;
	}

	/**
	 * Vraci, zda je nastaven flag Reserved.
	 */
	public boolean getReserved() {
		return reserved;
	}

	/**
	 * Vraci, zda je nastaven flag Don't Fragment.
	 */
	public boolean getDontFragment() {
		return dontFragment;
	}

	/**
	 * Vraci, zda je nastaven flag More Fragments.
	 */
	public boolean getMoreFragments() {
		return moreFragments;
	}

	/**
	 * Vraci hodnotu pole Fragment Offset.
	 */
	public int getFragmentOffset() {
		return packet.offset;
	}

	/**
	 * Vraci hodnotu pole Time to Live.
	 */
	public int getTimeToLive() {
		return packet.hop_limit;
	}

	/**
	 * Vraci hodnotu pole Protocol.
	 */
	public int getProtocol() {
		return packet.protocol;
	}

	/**
	 * Vraci hodnotu pole Header Checksum.
	 */
	public int getHeaderCheckSum() {
		return checksum;
	}

	/**
	 * Vraci hodnotu pole Source Address.
	 */
	public InetAddress getSourceAddress() {
		return packet.src_ip;
	}

	/**
	 * Vraci hodnotu pole Destination Address.
	 */
	public InetAddress getDestinationAddress() {
		return packet.dst_ip;
	}

	/**
	 * Vraci pole obsahujici volitelne parametry.
	 */
	public byte[] getOptions() {
		return options;
	}

	/**
	 * Vraci pole obsahujici data paketu.
	 */
	public byte[] getData() {
		return packet.data;
	}

	/**
	 * Nastavi paketu volitelne parametry.
	 * @param options Pole s volitelnymi parametry.
	 * @throws java.lang.Exception
	 */
	public void overrideOptions(byte[] options) throws Exception {
		if (this.options != null && this.options.length > 0) {
			return;
		}
		
		if (options.length % 4 != 0) {
			throw new Exception("Option list length must be multiple of 4");
		}

		headerLength += options.length;

		this.options = options;

	}
	
}
